package org.rcisoft.bus.wmsgoodsapply.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.bus.wmsgoods.entity.WmsGoods;
import org.rcisoft.bus.wmsgoodsapply.dto.AddWmsGoodsApplyDTO;
import org.rcisoft.bus.wmsgoodslog.dto.AddLogList;
import org.rcisoft.bus.wmsgoodslog.entity.WmsGoodsLog;
import org.rcisoft.bus.wmsgoodslog.service.WmsGoodsLogService;
import org.rcisoft.bus.wmsgoodsstock.entity.WmsGoodsStock;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.bus.wmsgoodsapply.dao.WmsGoodsApplyRepository;
import org.rcisoft.bus.wmsgoodsapply.entity.WmsGoodsApply;
import org.rcisoft.bus.wmsgoodsapply.service.WmsGoodsApplyService;
import org.rcisoft.bus.wmsgoodsstock.service.WmsGoodsStockService;

import org.rcisoft.sys.dictionary.dao.DictDataRepository;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class WmsGoodsApplyServiceImpl extends ServiceImpl<WmsGoodsApplyRepository,WmsGoodsApply>  implements WmsGoodsApplyService {
    @Autowired
    private DictDataRepository dictDataRepository;

    @Autowired
    private WmsGoodsStockService wmsGoodsStockRepositoryImpl;
    @Autowired
    private WmsGoodsLogService wmsGoodsLogServiceImpl;



    /**
    * 保存 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(WmsGoodsApply wmsGoodsApply){
        //增加操作
        //1.查库存表有没有对应的code  有则更新数量   没有则向库存表中插入
        //2.向日志表中添加对应的信息
        int line = baseMapper.insert(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel batchPersist(AddWmsGoodsApplyDTO addwmsGoodsApply) {
        //入库
        WmsGoodsApply wmsGoodsApply=new WmsGoodsApply();
        BeanUtils.copyProperties(addwmsGoodsApply,wmsGoodsApply);
        wmsGoodsApply.setIoType("0");
        wmsGoodsApply.setDealStatus("0");
        baseMapper.insert(wmsGoodsApply);

        //更新商品数量
        List<WmsGoods> boxList = addwmsGoodsApply.getBoxList();
        //空
        for (WmsGoods box : addwmsGoodsApply.getBoxList()) {
            WmsGoodsStock goodsStock = wmsGoodsStockRepositoryImpl.findById(addwmsGoodsApply.getBoxList().get(0).getBusinessId());
            if (goodsStock != null) {
                goodsStock.setGoodsCounts(goodsStock.getGoodsCounts() + box.getInCounts());
                wmsGoodsStockRepositoryImpl.updateCounts(goodsStock);
            } else {
                WmsGoodsStock newGoodsStock = new WmsGoodsStock();
                newGoodsStock.setGoodsId(addwmsGoodsApply.getBoxList().get(0).getBusinessId());
                newGoodsStock.setGoodsCounts(box.getInCounts());
                wmsGoodsStockRepositoryImpl.persist(newGoodsStock);
            }
            //日志
            AddLogList addLogList = new AddLogList();
            List<WmsGoodsLog> logList = new ArrayList<>();
            WmsGoodsStock wmsgoodsStock = wmsGoodsStockRepositoryImpl.findById(box.getBusinessId());
            WmsGoodsLog log = new WmsGoodsLog();
            log.setGoodsName(box.getGoodsName());
            log.setGoodsCode(box.getGoodsCode());
            WmsGoodsApply wmsGoodsApplyId=new WmsGoodsApply();
            wmsGoodsApplyId.setAppCode(addwmsGoodsApply.getAppCode());
            List<WmsGoodsApply> wmsGoodsApplyList= baseMapper.queryWmsGoodsApplys(wmsGoodsApplyId);
            log.setApplyId(wmsGoodsApplyList.get(0).getBusinessId());
            log.setGoodsCounts(box.getInCounts());
            log.setIoType("0");
            assert wmsgoodsStock != null;
            log.setUpdateCounts(wmsgoodsStock.getGoodsCounts());
            logList.add(log);
            addLogList.setBoxList(logList);
            wmsGoodsLogServiceImpl.batchPersist(addLogList);
        }
        return new CyPersistModel(1);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel batchOutPersist(AddWmsGoodsApplyDTO addwmsGoodsApply) {
        // 出库
        WmsGoodsApply wmsGoodsApplyOut = new WmsGoodsApply();
        BeanUtils.copyProperties(addwmsGoodsApply, wmsGoodsApplyOut);
        wmsGoodsApplyOut.setIoType("1");
        wmsGoodsApplyOut.setDealStatus("1");
        baseMapper.insert(wmsGoodsApplyOut);

        // 更新商品数量

        for (WmsGoods box : addwmsGoodsApply.getBoxList()) {
            WmsGoodsStock goodsStock = wmsGoodsStockRepositoryImpl.findById(box.getBusinessId());
            if (goodsStock != null) {
                if (box.getOutCounts() > goodsStock.getGoodsCounts()) {
                    throw new CyServiceException(1001,"库存数量不足");
                }
                goodsStock.setGoodsCounts(goodsStock.getGoodsCounts() - box.getOutCounts());
                wmsGoodsStockRepositoryImpl.updateCounts(goodsStock);
            }
            // 日志
            AddLogList addLogList = new AddLogList();
            List<WmsGoodsLog> logList = new ArrayList<>();
            WmsGoodsLog log = new WmsGoodsLog();
            log.setApplyId(box.getBusinessId());
            log.setGoodsCounts(box.getOutCounts());
            log.setIoType("1");
            assert goodsStock != null;
            log.setUpdateCounts(goodsStock.getGoodsCounts());
            logList.add(log);

            addLogList.setBoxList(logList);
            wmsGoodsLogServiceImpl.batchPersist(addLogList);
        }
        return new CyPersistModel(1);
    }


    @Override
    public String AppCode() {
        // 获取当前时间
        Date date = new Date();
        // 设置日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        // 格式化日期
        String formattedDate = sdf.format(date);
        String codePrefix = "RX";
        int OrderCodeCount = baseMapper.getOrderCodeCount(codePrefix+formattedDate) + 1;
        String formattedCount = String.format("%03d",OrderCodeCount);
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 编号前缀
        String AppCode = codePrefix + formattedDate + formattedCount;
        return AppCode;
    }
    /**
    * 删除 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(WmsGoodsApply wmsGoodsApply){
        int line = baseMapper.realDelete(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(WmsGoodsApply wmsGoodsApply){
        wmsGoodsApply.setDeleted();
        int line = baseMapper.deleteById(wmsGoodsApply);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                wmsGoodsApply.getBusinessId()+"的商品申请表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(WmsGoodsApply wmsGoodsApply){
        int line = baseMapper.updateById(wmsGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ wmsGoodsApply.getBusinessId()+"的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 商品申请表
    * @param id
    * @return
    */
    @Override
    public WmsGoodsApply findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Override
    public IPage<WmsGoodsApply> findAllByPagination(CyPageInfo<WmsGoodsApply> paginationUtility,
        WmsGoodsApply wmsGoodsApply){
        return baseMapper.queryWmsGoodsApplysPaged(paginationUtility,wmsGoodsApply);

    }

    /**
    * 查询list 商品申请表
    * @param wmsGoodsApply
    * @return
    */
    @Override
    public List<WmsGoodsApply> findAll(WmsGoodsApply wmsGoodsApply){
        return baseMapper.queryWmsGoodsApplys(wmsGoodsApply);

    }

    /**
    * 导出商品申请表
    * @return
    */
    @Override
    public List<WmsGoodsApply> export(WmsGoodsApply wmsGoodsApply) {
        List<WmsGoodsApply> wmsGoodsApplyList = baseMapper.queryWmsGoodsApplys(wmsGoodsApply);
        return wmsGoodsApplyList;
    }



    private String removeSymbol(String source) {
        String target = null;
        target = source.replaceAll("\\s*|\r|\n|\t","");
        if (target.contains(" ")) {
            target.replaceAll(" ","");
        }
        return target;
    }
    public Boolean isPositiveInteger(String figure) {
        // 如果输入为空，或者不是数字，则返回 false
        if (figure == null || figure.isEmpty()) {
            return false;
        }
        // 遍历字符串中的每个字符，检查是否都是数字
        for (char c : figure.toCharArray()) {
            if (!Character.isDigit(c)) {
                return false;
            }
        }
        // 将字符串转换为整数
        int num = Integer.parseInt(figure);
        // 如果大于 0 则为正整数，否则为非正整数
        return num > 0;
    }
}
